@@ -1,10 +1,16 @@
 /* draw.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_draw[] = "$Id: draw.c,v 2.54 1996/05/30 17:39:04 steve Exp $";
+char id_draw[] = "$Id: draw.c,v 2.75 1999/10/08 18:04:29 steve Exp $";
 
 #include "elvis.h"
 
+#if defined (GUI_WIN32)
+# define SLOPPY_ITALICS
+#else
+# undef SLOPPY_ITALICS
+#endif
+
 #if USE_PROTOTYPES
 static void insimage(CHAR *ch, char *font, int qty, int extent);
 static void delimage(CHAR *ch, char *font, int qty, int extent);
@@ -20,9 +26,10 @@ static void openmove(WINDOW win, long oldcol, long newcol, CHAR *image, long len
 #endif
 
 /* allocate a drawinfo struct, including the related arrays */
-DRAWINFO *drawalloc(rows, columns)
+DRAWINFO *drawalloc(rows, columns, top)
 	int	rows;	/* height of new window image */
 	int	columns;/* width of new window image */
+	MARK	top;	/* top of screen */
 {
 	DRAWINFO *newp;
 	int	 i;
@@ -37,6 +44,8 @@ DRAWINFO *drawalloc(rows, columns)
 	newp->curchar = (CHAR *)safealloc(rows * columns, sizeof(CHAR));
 	newp->curfont = (char *)safealloc(rows * columns, sizeof(char));
 	newp->offsets = (long *)safealloc(rows * columns, sizeof(long));
+	newp->topmark = markdup(top);
+	newp->bottommark = markdup(top);
 
 	/* clear the current image and the "new" image */
 	for (i = rows * columns; --i >= 0; )
@@ -67,6 +76,8 @@ void drawfree(di)
 	safefree(di->curchar);
 	safefree(di->curfont);
 	safefree(di->offsets);
+	markfree(di->topmark);
+	markfree(di->bottommark);
 	if (di->openimage)
 	{
 		safefree(di->openimage);
@@ -92,19 +103,11 @@ void drawexpose(win, top, left, bottom, right)
 	int	right;	/* right edge to be redrawn */
 {
 	int	row, column, same, base, nonblank;
-	MARKBUF first, last;
+	long	firstline, lastline;
 
 	assert(win != NULL && top >= 0 && left >= 0 && bottom < o_lines(win)
 		&& right < o_columns(win) && top <= bottom && left <= right);
 
-#if 0
-	/* if we must redraw anyway, then do nothing */
-	if (win->di->logic == DRAW_SCRATCH)
-	{
-		return;
-	}
-#endif
-
 	/* if this GUI has no moveto() function, then do nothing */
 	if (!gui->moveto)
 	{
@@ -114,7 +117,6 @@ void drawexpose(win, top, left, bottom, right)
 	/* for each row in the rectangle... */
 	for (row = top; row <= bottom; row++)
 	{
-#if 1
 		/* find the width of this row, ignoring trailing blanks */
 		for (nonblank = o_columns(win), base = o_columns(win) * row;
 		     nonblank > left
@@ -150,40 +152,41 @@ void drawexpose(win, top, left, bottom, right)
 		{
 			guiclrtoeol(win);
 		}
-#else
-		/* for each column in the row */
-		for (base = o_columns(win) * row + left, column = left;
-		     column <= right;
-		     base++, column++)
-		{
-			/* blot out elvis' idea of the current image.  This
-			 * will cause elvis to re-output it the next time
-			 * the image is updated.
-			 */
-			win->di->curfont[base] = '\0';
-		}
-#endif
 	}
 
-#if 1
 	/* leave the cursor in the right place */
 	guimoveto(win, win->di->curscol, win->di->cursrow);
-#endif
 
 	/* update the scrollbar, too */
 	if (gui->scrollbar)
 	{
 		if (win->md->move == dmnormal.move)
 		{
-			(*gui->scrollbar)(win->gw,
-				markline(marktmp(first, markbuffer(win->cursor), win->di->topline)) - 1,
-				markline(marktmp(last, markbuffer(win->cursor), win->di->bottomline)) - 1,
+			/* find line numbers of first and last lines shown */
+			firstline = markline(win->di->topmark) - 1;
+			lastline = markline(win->di->bottommark) - 1;
+
+			/* Some scrolling commands temporarily set bottommark
+			 * to the end of the buffer.  Ordinarily this causes
+			 * no problems, but if an expose event occurs before
+			 * the screen is redrawn, the the scrollbar's "thumb"
+			 * will briefly extend to the bottom of the scrollbar.
+			 * If we seem to be in that situation, then make a more
+			 * reasonable guess about the screen bottom line number.
+			 */
+			if (lastline - firstline >= o_lines(win))
+			{
+				lastline = firstline + o_lines(win) - 1;
+			}
+
+			/* update the scrollbar */
+			(*gui->scrollbar)(win->gw, firstline, lastline,
 				o_buflines(markbuffer(win->cursor)));
 		}
 		else
 		{
-			(*gui->scrollbar)(win->gw, win->di->topline,
-				win->di->bottomline, win->di->curnbytes);
+			(*gui->scrollbar)(win->gw, markoffset(win->di->topmark),
+				markoffset(win->di->bottommark), win->di->curnbytes);
 		}
 	}
 	guiflush();
@@ -257,7 +260,7 @@ static void delimage(ch, font, qty, extent)
 	int	qty;	/* number of characters to delete */
 	int	extent;	/* number of characters after the deletion point */
 {
-	register i;
+	register int i;
 
 	/* shift the characters */
 	for (i = 0; i < extent - qty; i++)
@@ -344,7 +347,7 @@ static void fillcell(ch, font, offset)
 
 		/* adjust limits, taking o_sidescroll into consideration */
 		thiscell -= o_columns(thiswin);
-		if (o_sidescroll(thiswin) == 0)
+		if (o_wrap(thiswin))
 		{
 			rightcol += o_columns(thiswin);
 		}
@@ -510,9 +513,9 @@ static void drawchar(p, qty, font, offset)
 		/* if this is a newline, then treat it as a bunch of spaces */
 		if (ch == '\n')
 		{
-			/* remember this line's width.  If a single "line" involves
-			 * multiple newlines, then the width of the last non-empty
-			 * phsyical line is kept.
+			/* remember this line's width.  If a single "line"
+			 * involves multiple newlines, then the width of the
+			 * last non-empty phsyical line is kept.
 			 */
 			if (thiscol != 0)
 			{
@@ -538,15 +541,23 @@ static void drawchar(p, qty, font, offset)
 			 * will be there.  This test must be made outside the
 			 * loop.
 			 */
-			if (offset == markoffset(thiswin->state->cursor)
-				&& thiscol % o_columns(thiswin) == 0)
-			{
-				thiscol = leftcol;
-			}
-			while (thiscol == leftcol || thiscell % o_columns(thiswin) != 0)
+			i = o_columns(thiswin);
+			if ((offset == markoffset(thiswin->state->cursor)
+				&& thiscol % i == 0) || thiscol == leftcol)
 			{
 				fillcell(' ', hifont, offset);
 			}
+			/* Note: thiscell may've changed since previous "if" */
+			if (thiscell % i != 0)
+			{
+				for (i -= thiscell % i; i > 0; i--)
+				{
+					di->newchar[thiscell] = ' ';
+					di->newfont[thiscell] = hifont;
+					di->offsets[thiscell] = offset;
+					thiscol++, thiscell++;
+				}
+			}
 
 			/* reset the virtual column number */
 			thiscol = 0;
@@ -567,11 +578,12 @@ static void drawchar(p, qty, font, offset)
 static void compareimage(win)
 	WINDOW	win;	/* window to be processed */
 {
+	typedef enum {NEW, BEFORE, AFTER} type_t;
 	int	i, j, k, diff, totdiff;
 	struct prevmatch_s
 	{
-		DRAWLINE *line;			/* ptr to new line's current info */
-		enum {NEW, BEFORE, AFTER} type;	/* line position, relative to any change */
+		DRAWLINE *line;	/* ptr to new line's current info */
+		type_t	 type;	/* line position, relative to any change */
 	}	*prev;
 
 	/* if we're supposed to redraw from scratch, then we won't be doing
@@ -730,50 +742,53 @@ printf("---------------------------------------------------------------------\n"
 static void updateimage(win)
 	WINDOW	win;	/* window to be updated */
 {
-	int	row, column, base, same, i, used;
+	int	row, column, same, used;
 	int	ncols = o_columns(win);
+	register int	  i, base;
+	register DRAWINFO *di = win->di;
+	register int	  rowXncols;
 
 	/* for each row... */
-	for (row = 0; row < o_lines(win); row++)
+	for (row = rowXncols = 0; row < o_lines(win); row++, rowXncols += ncols)
 	{
-		assert(win->di->newrow[row].insrows == 0 || win->di->logic != DRAW_SCRATCH);
+		assert(di->newrow[row].insrows == 0 || di->logic != DRAW_SCRATCH);
 
 		/* do we want to insert/delete rows? */
-		if (win->di->newrow[row].insrows != 0)
+		if (di->newrow[row].insrows != 0)
 		{
 			/* try to perform the insert/delete for the GUI */
 			guimoveto(win, 0, row);
-			if (guiscroll(win, win->di->newrow[row].insrows, (BOOLEAN)(maxrow < o_lines(win))))
+			if (guiscroll(win, di->newrow[row].insrows, (BOOLEAN)(maxrow < o_lines(win))))
 			{
 				/* perform the insert/delete on the "current" image */
-				if (win->di->newrow[row].insrows > 0)
+				if (di->newrow[row].insrows > 0)
 				{
 					/* insert some rows */
-					insimage(&win->di->curchar[row * ncols],
-						&win->di->curfont[row * ncols],
-						win->di->newrow[row].insrows * ncols,
+					insimage(&di->curchar[rowXncols],
+						&di->curfont[rowXncols],
+						di->newrow[row].insrows * ncols,
 						(scrollrows - row) * ncols);
 
 					/* shift amount of new lines is always 0 */
-					for (i = row; i < row + win->di->newrow[row].insrows; i++)
+					for (i = row; i < row + di->newrow[row].insrows; i++)
 					{
-						win->di->newrow[i].shiftright =
-							win->di->newrow[i].inschars = 0;
+						di->newrow[i].shiftright =
+							di->newrow[i].inschars = 0;
 					}
 				}
 				else
 				{
 					/* delete some rows */
-					delimage(&win->di->curchar[row * ncols],
-						&win->di->curfont[row * ncols],
-						-win->di->newrow[row].insrows * ncols,
+					delimage(&di->curchar[rowXncols],
+						&di->curfont[rowXncols],
+						-di->newrow[row].insrows * ncols,
 						(scrollrows - row) * ncols);
 
 					/* shift amount of new lines is always 0 */
-					for (i = maxrow + win->di->newrow[row].insrows; i < maxrow; i++)
+					for (i = maxrow + di->newrow[row].insrows; i < maxrow; i++)
 					{
-						win->di->newrow[i].shiftright =
-							win->di->newrow[i].inschars = 0;
+						di->newrow[i].shiftright =
+							di->newrow[i].inschars = 0;
 					}
 				}
 			}
@@ -782,47 +797,47 @@ static void updateimage(win)
 				/* don't do any more scrolling after this */
 				for (i = row; i < o_lines(win); i++)
 				{
-					win->di->newrow[i].insrows = 0;
+					di->newrow[i].insrows = 0;
 				}
 			}
 		}
 
 		/* perform shifting, if any */
-		if (win->di->newrow[row].shiftright != 0)
+		if (di->newrow[row].shiftright != 0)
 		{
 			/* see how many following rows have the same shift */
 			for (same = 1;
 			     gui->shiftrows && 
-			        row + same < win->di->rows - 1 &&
-				win->di->newrow[row].shiftright == win->di->newrow[row + same].shiftright;
+			        row + same < di->rows - 1 &&
+				di->newrow[row].shiftright == di->newrow[row + same].shiftright;
 			     same++)
 			{
 			}
 
 			/* make the GUI do its shift */
 			guimoveto(win, 0, row);
-			if (guishift(win, win->di->newrow[row].shiftright, same))
+			if (guishift(win, di->newrow[row].shiftright, same))
 			{
 				/* shift the current image, too */
 				for (i = row; i < row + same; i++)
 				{
-					if (win->di->newrow[i].shiftright > 0)
+					if (di->newrow[i].shiftright > 0)
 					{
 						/* shift right by inserting */
-						insimage(&win->di->curchar[i * ncols],
-							&win->di->curfont[i * ncols],
-							win->di->newrow[i].shiftright,
+						insimage(&di->curchar[i * ncols],
+							&di->curfont[i * ncols],
+							di->newrow[i].shiftright,
 							ncols);
 					}
 					else 
 					{
 						/* shift left by deleting */
-						delimage(&win->di->curchar[i * ncols],
-							&win->di->curfont[i * ncols],
-							-win->di->newrow[i].shiftright,
+						delimage(&di->curchar[i * ncols],
+							&di->curfont[i * ncols],
+							-di->newrow[i].shiftright,
 							ncols);
 					}
-					win->di->newrow[i].shiftright = 0;
+					di->newrow[i].shiftright = 0;
 				}
 			}
 		}
@@ -831,18 +846,29 @@ static void updateimage(win)
 		base = ncols * row;
 		for (used = ncols;
 		     used > 0
-			&& win->di->newchar[base + used - 1] == ' '
-			&& win->di->newfont[base + used - 1] == 'n';
+			&& di->newchar[base + used - 1] == ' '
+			&& di->newfont[base + used - 1] == 'n';
 		     used--)
 		{
 		}
 
+#ifdef SLOPPY_ITALICS
+		/* If the line ends with an italic character, then pretend
+		 * we're using one additional character.  This is so the last
+		 * italic character's right edge isn't clipped off.
+		 */
+		if (used < ncols && used >= 1 && di->newfont[base + used - 1] == 'i')
+		{
+			used++;
+		}
+#endif
+
 		/* look for mismatched segments */
 		for (column = 0; column < used; base += same, column += same)
 		{
 			/* if this cell matches, then skip to next */
-			if (win->di->newchar[base] == win->di->curchar[base]
-			 && win->di->newfont[base] == win->di->curfont[base])
+			if (di->newchar[base] == di->curchar[base]
+			 && di->newfont[base] == di->curfont[base])
 			{
 				same = 1;
 				continue;
@@ -856,43 +882,43 @@ static void updateimage(win)
 			 * we're still on the inserted blanks at the start
 			 * of the row.
 			 */
-			if (column >= win->di->newrow[row].shiftright
-			  && win->di->newrow[row].inschars != 0)
+			if (column >= di->newrow[row].shiftright
+			  && di->newrow[row].inschars != 0)
 			{
-				if (guishift(win, win->di->newrow[row].inschars, 1))
+				if (guishift(win, di->newrow[row].inschars, 1))
 				{
 					/* shift the current image, too */
-					if (win->di->newrow[row].inschars > 0)
+					if (di->newrow[row].inschars > 0)
 					{
 						/* shift right by inserting */
-						insimage(&win->di->curchar[row * ncols + column],
-							&win->di->curfont[row * ncols + column],
-							win->di->newrow[row].inschars,
+						insimage(&di->curchar[rowXncols + column],
+							&di->curfont[rowXncols + column],
+							di->newrow[row].inschars,
 							ncols - column);
 					}
 					else 
 					{
 						/* shift left by deleting */
-						delimage(&win->di->curchar[row * ncols + column],
-							&win->di->curfont[row * ncols + column],
-							-win->di->newrow[row].inschars,
+						delimage(&di->curchar[rowXncols + column],
+							&di->curfont[rowXncols + column],
+							-di->newrow[row].inschars,
 							ncols - column);
 					}
 
 					/* for other rows of the same line, insertions
 					 * will now appear to be shifts.
 					 */
-					for (i = row + 1; i < maxrow && win->di->newrow[i].lineoffset == win->di->newrow[row].lineoffset; i++)
+					for (i = row + 1; i < maxrow && di->newrow[i].lineoffset == di->newrow[row].lineoffset; i++)
 					{
-						win->di->newrow[i].shiftright += win->di->newrow[i].inschars;
-						win->di->newrow[i].inschars = 0;
+						di->newrow[i].shiftright += di->newrow[i].inschars;
+						di->newrow[i].inschars = 0;
 					}
 				}
 
 				/* we have now done all the inserting/deleting
 				 * that we're ever going to do for this row.
 				 */
-				win->di->newrow[row].inschars = 0;
+				di->newrow[row].inschars = 0;
 			}
 
 			/* See how many mismatched cells we can find with same
@@ -902,14 +928,18 @@ static void updateimage(win)
 			 */
 			for (same = 1, i = 0;
 			     column + same < used
-				&& (win->di->newchar[base + same] != win->di->curchar[base + same]
-					|| win->di->newfont[base + same] != win->di->curfont[base + same]
+				&& (di->newchar[base + same] != di->curchar[base + same]
+					|| di->newfont[base + same] != di->curfont[base + same]
 					|| i < gui->movecost)
-				&& win->di->newfont[base] == win->di->newfont[base + same];
+				&& (di->newfont[base] == di->newfont[base + same]
+#ifdef SLOPPY_ITALICS
+					|| (di->newfont[base + same - 1] == 'i' && di->newfont[base + same] == 'n' && di->newchar[base + same] == ' ')
+#endif
+										 );
 			     same++)
 			{
-				if (win->di->newchar[base + same] != win->di->curchar[base + same]
-					|| win->di->newfont[base + same] != win->di->curfont[base + same])
+				if (di->newchar[base + same] != di->curchar[base + same]
+					|| di->newfont[base + same] != di->curfont[base + same])
 				{
 					i = 0;
 				}
@@ -920,14 +950,32 @@ static void updateimage(win)
 			}
 			same -= i;
 
+#ifdef SLOPPY_ITALICS
+			/* if the change is in italics, and the preceding
+			 * characters were also italic, then include those
+			 * preceding characters in the redrawn span; otherwise 
+			 * they would be distorted by the new text.
+			 */
+			while (column > 0
+			    && di->newfont[base] == 'i'
+			    && di->newfont[base - 1] == 'i'
+			    && di->newchar[base - 1] != ' ')
+			{
+				base--;
+				column--;
+				same++;
+			}
+			guimoveto(win, column, row);
+#endif
+
 			/* write the cells to the GUI */
-			guidraw(win, win->di->newfont[base], &win->di->newchar[base], same);
+			guidraw(win, di->newfont[base], &di->newchar[base], same);
 
 			/* write the cells to the "current" image */
 			for (i = 0; i < same; i++)
 			{
-				win->di->curchar[base + i] = win->di->newchar[base + i];
-				win->di->curfont[base + i] = win->di->newfont[base + i];
+				di->curchar[base + i] = di->newchar[base + i];
+				di->curfont[base + i] = di->newfont[base + i];
 			}
 		}
 
@@ -935,8 +983,8 @@ static void updateimage(win)
 		base = ncols * row;
 		for (i = used;
 		     i < ncols
-			&& win->di->curchar[base + i] == ' '
-			&& win->di->curfont[base + i] == 'n';
+			&& di->curchar[base + i] == ' '
+			&& di->curfont[base + i] == 'n';
 		     i++)
 		{
 		}
@@ -959,8 +1007,8 @@ static void updateimage(win)
 			guiclrtoeol(win);
 			while (i < ncols)
 			{
-				win->di->curchar[base + i] = ' ';
-				win->di->curfont[base + i] = 'n';
+				di->curchar[base + i] = ' ';
+				di->curfont[base + i] = 'n';
 				i++;
 			}
 		}
@@ -974,8 +1022,9 @@ static void updateimage(win)
 static void genlastrow(win)
 	WINDOW	win;	/* window whose last row is to be generated */
 {
-	int	i, base;
+	int	i, j, base;
 	char	*scan;
+	CHAR	*cp;
 	char	buf[25];
 
 	/* were there any rows inserted or deleted? */
@@ -994,6 +1043,15 @@ static void genlastrow(win)
 			win->di->newchar[base + i] = ' ';
 			win->di->newfont[base + i] = 'n';
 		}
+		if (o_showname && o_bufname(markbuffer(win->cursor)))
+		{
+			for (i = 0, cp = o_bufname(markbuffer(win->cursor));
+			     i < o_columns(win) && *cp;
+			     i++, cp++)
+			{
+				win->di->newchar[base + i] = *cp;
+			}
+		}
 	}
 	else
 	{
@@ -1008,16 +1066,29 @@ static void genlastrow(win)
 		win->di->newmsg = False;
 	}
 
-	/* does the GUI have a status function? */
-	if (gui->status)
+	/* if colors changed, then we need to redraw last line from scratch */
+	if (win->di->logic != DRAW_NORMAL)
 	{
-		/* yes, call it with status info */
-		(*gui->status)(markline(win->state->cursor),
-			(*win->md->mark2col)(win, win->state->cursor, viiscmd(win)) + 1,
-			maplrnchar(o_modified(markbuffer(win->cursor)) ? '*' : ','),
-			win->state->modename);
+		guimoveto(win, 0, win->di->rows - 1);
+		guiclrtoeol(win);
+		for (i = o_columns(win); --i >= 0; )
+		{
+			win->di->curchar[base + i] = ' ';
+			win->di->curfont[base + i] = 'n';
+		}
 	}
-	else
+
+	/* does the GUI have a status function? */
+	if (!gui->status || !(*gui->status)(win->gw,
+#ifdef FEATURE_SHOWTAG
+		*win->cmdchars ? win->cmdchars : telabel(win->state->cursor),
+#else
+		win->cmdchars,
+#endif
+		markline(win->state->cursor),
+		(*win->md->mark2col)(win, win->state->cursor, viiscmd(win)) + 1,
+		maplrnchar(o_modified(markbuffer(win->cursor)) ? '*' : ','),
+		win->state->modename))
 	{
 		/* no, but maybe show status on the window's bottom row */
 
@@ -1028,7 +1099,7 @@ static void genlastrow(win)
 			for (i = base + o_columns(win) - 10; i < base + o_columns(win) - 3; i++)
 			{
 				win->di->newchar[i] = *scan++;
-				win->di->newfont[i] = 'E';	/* !!it E3: I prefer it emphasized */
+				win->di->newfont[i] = 'E';
 				if (!*scan)
 				{
 					scan = " ";
@@ -1060,6 +1131,36 @@ static void genlastrow(win)
 				win->di->newchar[i] = ' ';
 			win->di->newfont[i] = 'n';
 		}
+
+		/* if "showcmd" then show it */
+		if (o_showcmd(win))
+		{
+			i = base + o_columns(win) - 22 - QTY(win->cmdchars);
+			for (j = 0; win->cmdchars[j]; j++, i++)
+			{
+				win->di->newchar[i] = win->cmdchars[j];
+				win->di->newfont[i] = 'n';
+			}
+			for ( ; j < QTY(win->cmdchars) - 1; j++, i++)
+			{
+				win->di->newchar[i] = ' ';
+				win->di->newfont[i] = 'n';
+			}
+		}
+
+#ifdef FEATURE_SHOWTAG
+		/* if "showtag" and no messages, then show the tag */
+		if (o_showtag && win->di->newchar[base] == ' ')
+		{
+			for (i = base, cp = telabel(win->state->cursor);
+			     i < base + o_columns(win) - 22 && *cp;
+			     i++, cp++)
+			{
+				win->di->newchar[i] = *cp;
+				win->di->newfont[i] = 'n';
+			}
+		}
+#endif
 	}
 
 	/* if "showstack", then show it */
@@ -1117,7 +1218,7 @@ static BOOLEAN drawquick(win)
 	col = di->curscol;
 	if (o_number(win))
 		col += 8;
-	if (o_sidescroll(win) == 0)
+	if (o_wrap(win))
 		col %= di->columns;
 	else
 		col -= di->skipped;
@@ -1127,9 +1228,10 @@ static BOOLEAN drawquick(win)
 	    !win->md->canopt ||					/* display mode doesn't support optimization */
 	    di->logic != DRAW_NORMAL ||				/* last command made a subtle change */
 	    di->drawstate != DRAW_VISUAL ||			/* screen doesn't already show image */
+	    markbuffer(win->cursor)!=markbuffer(di->topmark) ||	/* cursor is in different buffer */
 	    di->curchgs != markbuffer(win->cursor)->changes ||	/* last command made a normal change */
-	    cursoff < di->topline ||				/* cursor off top of screen */
-	    cursoff >= di->bottomline ||			/* cursor off bottom of screen */
+	    cursoff < markoffset(di->topmark) ||		/* cursor off top of screen */
+	    cursoff >= markoffset(di->bottommark) ||		/* cursor off bottom of screen */
 	    col < 0 ||						/* cursor off left edge of screen */
 	    col >= di->columns ||				/* cursor off right edge of screen */
 	    win->seltop)					/* visually selecting text */
@@ -1138,13 +1240,11 @@ static BOOLEAN drawquick(win)
 	}
 
 	/* scan all rows at that column, looking for the cursor's offset */
-	for (i = 0, j = col;
-	     i < di->rows && di->offsets[j] < cursoff; /*!!! changed != to < */
-	     i++, j += di->columns)
+	for (i = 0, j = col; di->offsets[j] != cursoff; i++, j += di->columns)
 	{
+		if (i >= di->rows - 2)
+			return False;
 	}
-	if (i >= di->rows)
-		return False;
 
 	/* Hooray!  We can skip generating a new image */
 	di->cursrow = i;
@@ -1217,8 +1317,9 @@ void drawimage(win)
 	if (win->di->curbuf != markbuffer(win->cursor))
 	{
 		/* place the cursor's line at the center of the screen */
-		win->di->topline = markoffset(dispmove(win, -(o_lines(win) / 2), 0));
-		win->di->bottomline = o_bufchars(markbuffer(win->cursor));
+		markset(win->di->topmark, dispmove(win, -(o_lines(win) / 2), 0));
+		marksetbuffer(win->di->bottommark, markbuffer(win->cursor));
+		marksetoffset(win->di->bottommark, o_bufchars(markbuffer(win->cursor)));
 		win->di->logic = DRAW_CENTER;
 		win->di->curbuf = markbuffer(win->cursor);
 	}
@@ -1230,8 +1331,8 @@ void drawimage(win)
 	}
 
 	/* setup, and choose a starting point for the drawing */
-	next = (*win->md->setup)(marktmp(first, markbuffer(win->cursor), win->di->topline),
-		markoffset(win->cursor), marktmp(last, markbuffer(win->cursor), win->di->bottomline), win->mi);
+	next = (*win->md->setup)(win, win->di->topmark,
+		markoffset(win->cursor), win->di->bottommark, win->mi);
 	thiswin = win;
 	thiscell = 0;
 	thiscol = 0;
@@ -1243,7 +1344,7 @@ void drawimage(win)
 	{
 		win->di->curscol += 8; /* since line numbers will push text to the right */
 	}
-	if (o_sidescroll(win) == 0)
+	if (o_wrap(win))
 	{
 		/* we're doing traditional vi line wrapping */
 		leftcol = 0;
@@ -1295,11 +1396,11 @@ void drawimage(win)
 		wantcurs = win->di->rows;
 
 	/* draw each line until we reach the last row */
-	win->di->topline = markoffset(next);
+	markset(win->di->topmark, next);
 	for (row = 0; thiscell < maxcell || win->di->cursrow < 0 || (win->di->cursrow > wantcurs && markoffset(next) < o_bufchars(markbuffer(win->cursor))); )
 	{
 		/* set column limits, taking o_sidescroll into consideration */
-		if (o_sidescroll(win) == 0)
+		if (o_wrap(win))
 		{
 			rightcol = maxcell - thiscell;
 		}
@@ -1367,9 +1468,27 @@ void drawimage(win)
 			}
 		}
 	}
-	win->di->bottomline = markoffset(next);
+	markset(win->di->bottommark, next);
 	assert(linesshown > 0 && linesshown <= maxrow);
 
+	/* update the scrollbar */
+	if (gui->scrollbar)
+	{
+		if (win->md->move == dmnormal.move)
+		{
+			(*gui->scrollbar)(win->gw,
+				markline(marktmp(first, markbuffer(win->cursor), win->di->newline[0].start)) - 1,
+				markline(marktmp(last, markbuffer(win->cursor), markoffset(next))) - 1,
+				o_buflines(markbuffer(win->cursor)));
+		}
+		else
+		{
+			(*gui->scrollbar)(win->gw, win->di->newline[0].start,
+				markoffset(next), o_bufchars(markbuffer(next)));
+		}
+		guiflush();
+	}
+
 	/* compute insert/delete info */
 	compareimage(win);
 
@@ -1383,8 +1502,8 @@ void drawimage(win)
 	 * efficiently next time.
 	 */
 	win->di->logic = nextlogic;
-	win->di->topline = win->di->newline[0].start;
-	win->di->bottomline = markoffset(next);
+	marksetoffset(win->di->topmark, win->di->newline[0].start);
+	markset(win->di->bottommark, next);
 	win->di->nlines = linesshown;
 	win->di->curnbytes = o_bufchars(markbuffer(next));
 	win->di->curchgs = markbuffer(next)->changes;
@@ -1398,23 +1517,6 @@ void drawimage(win)
 
 	/* leave the cursor in the right place */
 	guimoveto(win, win->di->curscol, win->di->cursrow);
-
-	/* update the scrollbar */
-	if (gui->scrollbar)
-	{
-		if (win->md->move == dmnormal.move)
-		{
-			(*gui->scrollbar)(win->gw,
-				markline(marktmp(first, markbuffer(win->cursor), win->di->topline)) - 1,
-				markline(marktmp(last, markbuffer(win->cursor), win->di->bottomline)) - 1,
-				o_buflines(markbuffer(win->cursor)));
-		}
-		else
-		{
-			(*gui->scrollbar)(win->gw, win->di->topline,
-				win->di->bottomline, win->di->curnbytes);
-		}
-	}
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1642,6 +1744,10 @@ static CHAR	*openimage;	/* buffer, holds new line image */
 static long	opensize;	/* size of openimage */
 static BOOLEAN	opencursfound;	/* has the cursor's cell been found yet? */
 static BOOLEAN	openskipping;	/* has the line containing the cursor been completed? */
+static BOOLEAN	openselect;	/* currently outputting highlighted text? */
+static long	openseltop;	/* offset of top of highlighted region */
+static long	openselbottom;	/* offset of bottom of highlighted region */
+
 static void openchar(p, qty, font, offset)
 	CHAR	*p;	/* characters to be output */
 	long	qty;	/* number of characters */
@@ -1681,6 +1787,36 @@ static void openchar(p, qty, font, offset)
 			opencursfound = True;
 		}
 
+		/* expand the size of the buffer, if necessary */
+		if (opencnt + 2 >= opensize)
+		{
+			newp = (CHAR *)safealloc((int)(opensize + 80), sizeof(CHAR));
+			memcpy(newp, openimage, (size_t)opensize);
+			safefree(openimage);
+			openimage = newp;
+			opensize += 80;
+		}
+
+		/* if starting the highlighted text, output "*[" */
+		if (offset >= openseltop && offset <= openselbottom)
+		{
+			if (!openselect)
+			{
+				openimage[opencnt++] = '*';
+				openimage[opencnt++] = '[';
+				openselect = True;
+			}
+		}
+		else
+		{
+			if (openselect)
+			{
+				openimage[opencnt++] = ']';
+				openimage[opencnt++] = '*';
+				openselect = False;
+			}
+		}
+
 		/* newline is generally ignored */
 		if (ch == '\n')
 		{
@@ -1695,16 +1831,6 @@ static void openchar(p, qty, font, offset)
 			continue;
 		}
 
-		/* expand the size of the buffer, if necessary */
-		if (opencnt >= opensize)
-		{
-			newp = (CHAR *)safealloc((int)(opensize + 80), sizeof(CHAR));
-			memcpy(newp, openimage, (size_t)opensize);
-			safefree(openimage);
-			openimage = newp;
-			opensize += 80;
-		}
-
 		/* store the character */
 		openimage[opencnt++] = ch;
 	}
@@ -1792,19 +1918,27 @@ void drawopenedit(win)
 	/* else we're editing the same line as last time */
 
 	/* generate the new line image */
-	opencursfound = openskipping = False;
+	opencursfound = openskipping = openselect = False;
 	opencnt = 0;
 	openoffsetcurs = markoffset(win->state->cursor);
 	openimage = (CHAR *)safealloc(80, sizeof(CHAR));
 	opensize = 80;
+	openseltop = openselbottom = INFINITY;
 	if (win->state->acton)
 	{
-		curline = (*dmnormal.setup)(curline, openoffsetcurs, win->state->cursor, win->mi);
+		curline = (*dmnormal.setup)(win, curline, openoffsetcurs, win->state->cursor, win->mi);
 		curline = (*dmnormal.image)(win, curline, win->mi, openchar);
 	}
 	else
 	{
-		curline = (*win->md->setup)(curline, openoffsetcurs, win->state->cursor, win->mi);
+		if (win->seltop)
+		{
+			openseltop = markoffset(win->seltop);
+			openselbottom = markoffset(win->selbottom);
+			curcol += 2; /* for the *[ marker */
+		}
+		curline = (*win->md->setup)(win, curline, openoffsetcurs,
+						win->state->cursor, win->mi);
 		curline = (*win->md->image)(win, curline, win->mi, openchar);
 	}
 
@@ -1924,3 +2058,38 @@ void drawextext(win, text, len)
 		}
 	}
 }
+
+/* This is like drawextext, except that here we're careful about making
+ * control characters (other than newline) visible.
+ */
+void drawexlist(win, text, len)
+	WINDOW	win;	/* window where ex output should be drawn */
+	CHAR	*text;	/* text to be output */
+	int	len;	/* length of text */
+{
+	CHAR	buf[2];
+	int	from, to;
+
+	/* divide the text into segments of all-printable characters, or
+	 * or individual non-printable characters.
+	 */
+	for (from = to = 0; to < len; to++)
+	{
+		if (text[to] != '\n' && iscntrl(text[to]))
+		{
+			/* draw the last printable segment, if any */
+			if (from < to)
+				drawextext(win, &text[from], to - from);
+			from = to + 1;
+
+			/* draw this control char in a printable format */
+			buf[0] = '^';
+			buf[1] = text[to] ^ '@';
+			drawextext(win, buf, 2);
+		}
+	}
+
+	/* if we ended with a printable segment, then draw it now */
+	if (from < to)
+		drawextext(win, &text[from], to - from);
+}
