@@ -1,7 +1,7 @@
 /* event.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_event[] = "$Id: event.c,v 2.28 1996/05/23 00:24:13 steve Exp $";
+char id_event[] = "$Id: event.c,v 2.53 1999/02/17 22:54:22 steve Exp $";
 
 #include "elvis.h"
 
@@ -11,12 +11,72 @@ char id_event[] = "$Id: event.c,v 2.28 1996/05/23 00:24:13 steve Exp $";
 # define WATCH(x)	x
 #endif
 
+#ifdef NDEBUG
+# define USUAL_SUSPECTS
+#else
+# define USUAL_SUSPECTS {WINDOW w; for (w = windows; w; w = w->next){ \
+	if (!w->state) continue; \
+	assert(markbuffer(w->state->top) == markbuffer(w->state->cursor)); \
+	assert(markbuffer(w->state->bottom) == markbuffer(w->state->cursor)); \
+	assert(markoffset(w->state->top) <= markoffset(w->state->cursor)); \
+	assert(markoffset(w->state->bottom) >= markoffset(w->state->cursor)); \
+    } }
+#endif
+
+#if USE_PROTOTYPES
+static void setcursor(WINDOW win, long offset, BOOLEAN clean);
+#endif
 
 /* This counts the number of events that have been processed.  In particular,
  * if eventcounter==0 then we know we're still performing initialization.
  */
 long eventcounter;
 
+/* Some events move the cursor. E.g., clicking the mouse or moving the
+ * scrollbar's thumb.  If the window is in input mode, then there may be
+ * some superfluous text between the cursor and the end of the edit region
+ * (usually marked with a '$') which needs to be deleted before the cursor
+ * can be moved.
+ *
+ * This function detects those situations, and performs the necessary steps.
+ * The eventXXX functions should always use this function to set the cursor
+ * position.
+ */
+static void setcursor(win, offset, clean)
+	WINDOW	win;	/* window whose cursor should be moved */
+	long	offset;	/* new offset for the cursor */
+	BOOLEAN	clean;	/* definitely delete superfluous text */
+{
+	/* if in input mode, and there are superfluous characters after the
+	 * cursor (probably the end is marked with a '$' character) then we
+	 * may need to delete those characters.  Exception: If the new
+	 * location is still in the same edit region, then we don't need to
+	 * worry about it.
+	 */
+	if ((win->state->flags & ELVIS_REGION) == ELVIS_REGION
+	 && markoffset(win->state->bottom) != markoffset(win->state->cursor)
+	 && (clean
+	  || offset < markoffset(win->state->top)
+	  || markoffset(win->state->bottom) <= offset))
+	{
+		/* tweak offset to compensate for the following bufreplace() */
+		if (offset > markoffset(win->state->bottom))
+			offset -= (markoffset(win->state->bottom) - markoffset(win->state->cursor));
+
+		/* delete the superfluous text */
+		bufreplace(win->state->cursor, win->state->bottom, NULL, 0L);
+	}
+	
+	/* move the cursor, and maybe adjust the edit region too */
+	marksetoffset(win->state->cursor, offset);
+	if (offset < markoffset(win->state->top)
+		 || markoffset(win->state->bottom) <= offset)
+	{
+		marksetoffset(win->state->top, offset);
+		marksetoffset(win->state->bottom, offset);
+	}
+}
+
 /* This function creates a WINDOW -- an internal data structure
  * used to describe the characteristics of a window.  The GUI
  * calls this function when an elvis window suddenly pops into
@@ -39,6 +99,7 @@ BOOLEAN eventcreate(gw, guivals, name, rows, columns)
 	WINDOW	win;
 
 WATCH(fprintf(stderr, "eventcreate(..., name=\"%s\", rows=%d, columns=%d)\n", name, rows, columns));
+	USUAL_SUSPECTS
 	eventcounter++;
 
 	/* find/create a buffer with the requested name */
@@ -51,6 +112,10 @@ WATCH(fprintf(stderr, "eventcreate(..., name=\"%s\", rows=%d, columns=%d)\n", na
 	/* create a WINDOW */
 	win = winalloc(gw, guivals, buf, rows, columns);
 
+#ifndef NDEBUG
+	if (gui->moveto) USUAL_SUSPECTS
+#endif
+
 	return (BOOLEAN)(win != NULL);
 }
 
@@ -62,6 +127,9 @@ void eventdestroy(gw)
 	BUFFER	msgq;
 
 WATCH(fprintf(stderr, "eventdestroy(...)\n"));
+#ifndef NDEBUG
+	if (gui->moveto) USUAL_SUSPECTS
+#endif
 	eventcounter++;
 
 	/* find the window */
@@ -81,6 +149,7 @@ WATCH(fprintf(stderr, "eventdestroy(...)\n"));
 		winoptions(win);
 		msgflush();
 	}
+	USUAL_SUSPECTS
 }
 
 /* Change the size of an existing window.  This doesn't automatically redraw
@@ -93,15 +162,26 @@ void eventresize(gw, rows, columns)
 {
 	WINDOW	win;
 
+	USUAL_SUSPECTS
 WATCH(fprintf(stderr, "eventresize(..., rows=%d, columns=%d\n", rows, columns));
 	eventcounter++;
 
+	/* if too small, then ignore the change */
+	if (rows < 2 || columns < 30)
+	{
+		return; /* False? */
+	}
+
 	/* find the window */
 	win = winofgw(gw);
 	assert(win);
 
 	/* resize it */
 	winresize(win, rows, columns);
+
+	/* announce the new size */
+	msg(MSG_INFO, "[dd]$1 rows, $2 columns", (long)rows, (long)columns);
+	USUAL_SUSPECTS
 }
 
 /* Replace the current buffer with some other buffer, for the WINDOW
@@ -117,6 +197,7 @@ void eventreplace(gw, freeold, name)
 	BUFFER	buf;
 
 WATCH(fprintf(stderr, "eventreplace(..., freeold=%s, name=\"%s\")\n", freeold?"True":"False", name));
+	USUAL_SUSPECTS
 	eventcounter++;
 
 	/* find/create a buffer with the requested name */
@@ -132,6 +213,7 @@ WATCH(fprintf(stderr, "eventreplace(..., freeold=%s, name=\"%s\")\n", freeold?"T
 
 	/* replace its buffer */
 	winchgbuf(win, buf, True);
+	USUAL_SUSPECTS
 }
 
 /* Redraw a portion of a window. */
@@ -145,13 +227,18 @@ void eventexpose(gw, top, left, bottom, right)
 	WINDOW	win = winofgw(gw);
 
 WATCH(fprintf(stderr, "eventexpose(..., top=%d, left=%d, bottom=%d, right=%d)\n", top, left, bottom, right));
+	USUAL_SUSPECTS
 	eventcounter++;
 
+	/* permit the bell to ring */
+	guibeep(NULL);
+
 	if (top < 0) top = 0;
 	if (left < 0) left = 0;
 	if (bottom >= o_lines(win)) bottom = o_lines(win) - 1;
 	if (right >= o_columns(win)) right = o_columns(win) - 1;
 	drawexpose(win, top, left, bottom, right);
+	USUAL_SUSPECTS
 }
 
 /* Makes sure that a window shows a current image of its buffer's
@@ -164,11 +251,16 @@ ELVCURSOR eventdraw(gw)
 	WINDOW	win = winofgw(gw);
 
 WATCH(fprintf(stderr, "eventdraw(...)\n"));
+	USUAL_SUSPECTS
 	assert(win && win->state);
 	eventcounter++;
 
 	/* flush any messages */
-	msgflush();
+	if (!makeflag)
+		msgflush();
+
+	/* permit the bell to ring */
+	guibeep(NULL);
 
 	/* call drawopenedit for open-mode windows, or drawimage for visual-mode */
 	if (win->state->flags & ELVIS_BOTTOM)
@@ -189,6 +281,15 @@ WATCH(fprintf(stderr, "eventdraw(...)\n"));
 		/* draw the screen */
 		drawimage(win);
 	}
+
+	/* After user hits <Enter>, reset the makeflag */
+	if (makeflag && morehit)
+	{
+		makeflag = morehit = False;
+		msgflush();
+	}
+
+	USUAL_SUSPECTS
 	return (win->state ? (*win->state->shape)(win) : CURSOR_NONE);
 }
 
@@ -203,10 +304,12 @@ ELVCURSOR eventfocus(gw)
 	WINDOW	win = winofgw(gw);
 
 WATCH(fprintf(stderr, "eventfocus(...)\n"));
+	USUAL_SUSPECTS
 	assert(win && win->state);
 	eventcounter++;
 
 	winoptions(win);
+	USUAL_SUSPECTS
 	return (*win->state->shape)(win);
 }
 
@@ -231,10 +334,24 @@ long eventclick(gw, row, column, what)
 	long	offset;
 
 WATCH(fprintf(stderr, "eventclick(..., row=%d, column=%d,...)\n", row, column));
+	USUAL_SUSPECTS
 	eventcounter++;
 
 	/* reset the poll frequency counter */
 	guipoll(True);
+	bufmsgtype = MSG_STATUS;
+
+	/* make this window be the current window */
+	winoptions(win);
+
+	/* if the window is showing "Hit <Enter> to continue" then any click
+	 * is treated as an <Enter> keystoke.
+	 */
+	if (!strcmp(win->state->modename, "More"))
+	{
+		(void)eventkeys(gw, toCHAR("\n"), 1);
+		return markoffset(win->cursor);
+	}
 
 	/* for some operations, the position doesn't matter */
 	switch (what)
@@ -242,12 +359,14 @@ WATCH(fprintf(stderr, "eventclick(..., row=%d, column=%d,...)\n", row, column));
 	  case CLICK_CANCEL:
 		vinfbuf.command = ELVCTRL('[');
 		(void)v_visible(win, &vinfbuf);
+		USUAL_SUSPECTS
 		return 0;
 
 	  case CLICK_YANK:
 		/* if no text is marked, then fail */
 		if (!win->seltop)
 		{
+			USUAL_SUSPECTS
 			return -1;
 		}
 
@@ -276,6 +395,7 @@ WATCH(fprintf(stderr, "eventclick(..., row=%d, column=%d,...)\n", row, column));
 			cutyank((_CHAR_)'>', win->seltop, &tmp, win->seltype, False);
 			break;
 		}
+		USUAL_SUSPECTS
 		return 0;
 
 	  case CLICK_PASTE:
@@ -283,34 +403,36 @@ WATCH(fprintf(stderr, "eventclick(..., row=%d, column=%d,...)\n", row, column));
 		vinfbuf.command = ELVCTRL('[');
 		(void)v_visible(win, &vinfbuf);
 
-#if 1
+		/* delete any superfluous text after the cursor */
+		setcursor(win, markoffset(win->state->cursor), True);
+
 		/* set the buffer's "willdo" flag so this paste is undoable */
-		bufwilldo(win->state->cursor);
+		if (!win->state->pop)
+			bufwilldo(win->state->cursor, True);
 
 		/* paste the text */
 		newcurs = cutput((_CHAR_)'<', win, win->state->cursor, False, True, True);
 		if (newcurs)
 		{
-			marksetoffset(win->state->cursor, markoffset(newcurs) + 1);
-			marksetoffset(win->state->bottom, markoffset(newcurs) + 1);
+			setcursor(win, markoffset(newcurs) + 1, True);
 		}
-#else
-		vinfbuf.command = '@';
-		vinfbuf.key2 = '<';
-		(void)v_at(win, &vinfbuf);
-#endif
+		USUAL_SUSPECTS
 		return 0;
 
 	  case CLICK_TAG:
 		/* simulate a <Control-]> keystroke */
 		vinfbuf.command = ELVCTRL(']');
+		tmp = *win->state->cursor;
 		(void)v_tag(win, &vinfbuf);
+		USUAL_SUSPECTS
 		return 0;
 		
 	  case CLICK_UNTAG:
 	  	/* simulate a <Control-T> keystroke */
 	  	vinfbuf.command = ELVCTRL('T');
-	  	(void)v_tag(win, &vinfbuf);
+		tmp = *win->state->cursor;
+		(void)v_tag(win, &vinfbuf);
+		USUAL_SUSPECTS
 	  	return 0;
 
 	  default:
@@ -326,6 +448,7 @@ WATCH(fprintf(stderr, "eventclick(..., row=%d, column=%d,...)\n", row, column));
 	 || row < 0 || row >= o_lines(win) - 1
 	 || column < 0 || column >= o_columns(win))
 	{
+		USUAL_SUSPECTS
 		return -1;
 	}
 
@@ -333,12 +456,17 @@ WATCH(fprintf(stderr, "eventclick(..., row=%d, column=%d,...)\n", row, column));
 	offset =  win->di->offsets[row * o_columns(win) + column];
 	if (offset < 0 || offset >= o_bufchars(markbuffer(win->state->cursor)))
 	{
+		USUAL_SUSPECTS
 		return -1;
 	}
 
-	/* perform the requested operation */
-	marksetoffset(win->state->cursor, offset);
+	/* move the cursor to the click-on cell */
 	win->wantcol = column;
+	if (o_number(win))
+		win->wantcol -= 8;
+	setcursor(win, offset, False);
+
+	/* perform the requested operation */
 	switch (what)
 	{
 	  case CLICK_MOVE:
@@ -384,6 +512,7 @@ WATCH(fprintf(stderr, "eventclick(..., row=%d, column=%d,...)\n", row, column));
 		/*NOTREACHED*/
 		;
 	}
+	USUAL_SUSPECTS
 	return offset;
 }
 
@@ -397,17 +526,22 @@ MAPSTATE eventkeys(gw, key, nkeys)
 	CHAR	*key;	/* array of ASCII characters from key */
 	int	nkeys;	/* number of ASCII characters */
 {
+	MAPSTATE mapstate;
 WATCH(fprintf(stderr, "eventkeys(..., key={%d, ...}, nkeys=%d)\n", key[0], nkeys));
+	USUAL_SUSPECTS
 	eventcounter++;
 
 	/* reset the poll frequency counter */
 	guipoll(True);
+	bufmsgtype = MSG_STATUS;
 
 	/* Make this the default window, if it isn't already */
 	winoptions(winofgw(gw));
 
 	/* send the keys through the mapper */
-	return mapdo(key, nkeys);
+	mapstate = mapdo(key, nkeys);
+	USUAL_SUSPECTS
+	return mapstate;
 }
 
 
@@ -426,7 +560,10 @@ BOOLEAN eventscroll(gw, scroll, count, denom)
 	WINDOW	win = winofgw(gw);
 	VIINFO	cmd;
 	RESULT	result;
+	long	origoffset;
+	long	newoffset;
 
+	USUAL_SUSPECTS
 	assert(win != NULL && (count > 0 || (count == 0 && scroll == SCROLL_PERCENT)));
 	eventcounter++;
 
@@ -437,9 +574,13 @@ BOOLEAN eventscroll(gw, scroll, count, denom)
 	if (win->state->flags & ELVIS_BOTTOM)
 	{
 		msg(MSG_WARNING, "not while in open mode");
+		USUAL_SUSPECTS
 		return False;
 	}
 
+	/* remember the cursor's original offset */
+	origoffset = markoffset(win->state->cursor);
+
 	/* build & execute a vi command */
 	cmd.count = count;
 	switch (scroll)
@@ -472,29 +613,45 @@ BOOLEAN eventscroll(gw, scroll, count, denom)
 	  case SCROLL_PERCENT:
 		if (count > 0)
 		{
-			if (win->md->move == dmnormal.move && denom > 0)
+			if (win->md->move == dmnormal.move && denom != 0)
 			{
 				cmd.command = 'G';
-				cmd.count = o_buflines(markbuffer(win->cursor)) * count / denom;
-				if (cmd.count == 0)
-					cmd.count = 1;
+				cmd.count = 1 + o_buflines(markbuffer(win->cursor)) * count / denom;
+				if (cmd.count > o_buflines(markbuffer(win->cursor)))
+					cmd.count = o_buflines(markbuffer(win->cursor));
 				result = m_absolute(win, &cmd);
 			}
+			else if (o_bufchars(markbuffer(win->cursor)) == denom)
+			{
+				/* Move the cursor by setting its offset */
+				marksetoffset(win->cursor, count);
+				result = RESULT_COMPLETE;
+			}
 			else
 			{
+				/* Prescale so we don't overflow 32-bit math */
+				while (0x7fffffff / o_bufchars(markbuffer(win->cursor)) < count)
+				{
+					count >>= 1;
+					denom >>= 1;
+				}
+
+				/* Move the cursor by setting its offset */
 				if (o_bufchars(markbuffer(win->cursor)) != 0)
 					marksetoffset(win->cursor, (o_bufchars(markbuffer(win->cursor)) - 1) * count / denom);
 				result = RESULT_COMPLETE;
 			}
 			if (result == RESULT_COMPLETE)
 			{
+				/* Scroll the cursor's line to top of window */
 				cmd.count = cmd.count2 = 0L;
 				cmd.command = 'z';
 				cmd.key2 = '+';
 				result = m_z(win, &cmd);
 				if (result == RESULT_COMPLETE)
 				{
-					cmd.command = '$';
+					cmd.count = win->wantcol + 1;
+					cmd.command = ELVCTRL('|');
 					result = m_column(win, &cmd);
 				}
 			}
@@ -513,6 +670,17 @@ BOOLEAN eventscroll(gw, scroll, count, denom)
 		break;
 	}
 
+	/* Now we need to get clever about moving the cursor, due to the
+	 * possibility of superfluous text after the old cursor position
+	 * if we happened to be in input mode.  Force the cursor back to
+	 * its old offset, and then use setcursor() to move it to its new
+	 * offset in an input-mode-sensitive way.
+	 */
+	newoffset = markoffset(win->state->cursor);
+	marksetoffset(win->state->cursor, origoffset);
+	setcursor(win, newoffset, True);
+
+	USUAL_SUSPECTS
 	return False;
 }
 
@@ -521,18 +689,29 @@ BOOLEAN eventscroll(gw, scroll, count, denom)
 void eventsuspend()
 {
 	BUFFER	buf;
+	WINDOW	win;
 	eventcounter++;
 
+	USUAL_SUSPECTS
+
 	/* if autowrite is False, then do nothing */
 	if (!o_autowrite)
 		return;
 
+	/* If there is superfluous text after any cursor, delete it */
+	for (win = winofbuf(NULL, NULL); win; win = winofbuf(win, NULL))
+	{
+		setcursor(win, markoffset(win->state->cursor), True);
+	}
+
 	/* save all user buffers */
-	for (buf = buffers; buf; buf = buf->next)
+	for (buf = elvis_buffers; buf; buf = buf->next)
 	{
 		if (!o_internal(buf))
 			bufsave(buf, False, False);
 	}
+
+	USUAL_SUSPECTS
 }
 
 /* This function is called to execute ex command lines.  This may be a result
@@ -547,19 +726,32 @@ void eventex(gw, cmd, safer)
 {
 	BOOLEAN	origsafer;
 
+	eventcounter++;
+	USUAL_SUSPECTS
+
 	/* reset the poll frequency counter */
 	guipoll(True);
+	bufmsgtype = MSG_STATUS;
 
 	/* temporarily set the "safer" option appropriately */
 	origsafer = o_safer;
-	o_safer |= safer;
+	if (safer)
+		o_safer = True;
 
-	/* Make this the default window, if it isn't already */
+	/* Make this the default thing, if it isn't already */
 	winoptions(winofgw(gw));
 
+	/* If there is superfluous text after the cursor, delete it */
+	setcursor(windefault, markoffset(windefault->state->cursor), True);
+
 	/* execute the command */
-	exstring(windefault, toCHAR(cmd));
+	exstring(windefault, toCHAR(cmd), "gui");
+
+	/* update the state as though a key had just been pressed */
+	statekey((_CHAR_)-1);
 
 	/* restore the "safer" option */
 	o_safer = origsafer;
+
+	USUAL_SUSPECTS
 }
