@@ -1,7 +1,7 @@
 /* ex.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_ex[] = "$Id: ex.c,v 2.147 1998/11/21 01:36:11 steve Exp $";
+char id_ex[] = "$Id: ex.c,v 2.153 1999/10/08 18:03:03 steve Exp $";
 
 #include "elvis.h"
 
@@ -122,12 +122,14 @@ static struct
 /*c   */{"change",	EX_CHANGE,	ex_append,	a_Range | a_Bang | a_Count | a_Text | q_Undo | q_Ex			},
 /*cha */{"chdir",	EX_CD,		ex_cd,		a_Bang | a_File | q_Exrc | q_Unsafe					},
 /*ca  */{"calculate",	EX_CALC,	ex_comment,	a_Cmds | q_Exrc								},
+/*cas */{"case",	EX_CASE,	ex_case,	a_Lhs | a_Cmds | q_Exrc							},
 /*cc  */{"cc",		EX_CC,		ex_make,	a_Bang | a_Rhs | q_Unsafe | q_SwitchB					},
 /*cd  */{"cd",		EX_CD,		ex_cd,		a_Bang | a_File | q_Exrc | q_Unsafe					},
 /*cl  */{"close",	EX_CLOSE,	ex_xit,		a_Bang | q_MayQuit							},
 /*co  */{"copy",	EX_COPY,	ex_move,	a_Range | a_Target | a_Pflag | q_Autop | q_Undo				},
 /*col */{"color",	EX_COLOR,	ex_color,	a_Rhs | q_Exrc | q_Custom						},
 /*d   */{"delete",	EX_DELETE,	ex_delete,	a_Range | a_Buffer | a_Count | a_Pflag | q_Undo | q_Autop		},
+/*de  */{"default",	EX_DEFAULT,	ex_case,	a_Cmds | q_Exrc								},
 /*di  */{"display",	EX_DISPLAY,	ex_display,	a_Rhs									},
 /*dig */{"digraph",	EX_DIGRAPH,	ex_digraph,	a_Bang | a_Rhs | q_Exrc | q_Custom					},
 /*do  */{"do",		EX_DO,		ex_do,		a_Cmds | q_Exrc			 					},
@@ -192,6 +194,7 @@ static struct
 /*sta */{"stag",	EX_STAG,	ex_tag,		a_Rhs									},
 /*stac*/{"stack",	EX_STACK,	ex_stack,	d_None									},
 /*su  */{"suspend",	EX_SUSPEND,	ex_suspend,	a_Bang | q_Unsafe							},
+/*sw  */{"switch",	EX_SWITCH,	ex_switch,	a_Cmds | q_Exrc								},
 /*t   */{"to",		EX_COPY,	ex_move,	a_Range | a_Target | a_Pflag | q_Autop | q_Undo				},
 /*ta  */{"tag",		EX_TAG,		ex_tag,		a_Bang | a_Rhs | q_Unsafe | q_SwitchB					},
 /*th  */{"then",	EX_THEN,	ex_then,	a_Cmds | q_Exrc			 					},
@@ -226,6 +229,7 @@ static struct
 static int 	globaldepth;
 
 
+
 /* This function discards info from an EXINFO struct.  The struct itself
  * is not freed, since it is usually just a local variable in some function.
  */
@@ -934,11 +938,27 @@ static void parseplus(refp, xinf, flags)
 	/* skip the '+' */
 	scannext(refp);
 
-	/* collect following characters up to next whitespace or '|' */
-	while (*refp && !isspace(**refp) && **refp != '|')
+	/* collect the chars of the command string */
+	if (*refp && **refp == '"')
 	{
-		buildCHAR(&xinf->lhs, **refp);
+		/* collect all characters within quotes */
 		scannext(refp);
+		while (*refp && **refp != '\n' && **refp != '"')
+		{
+			buildCHAR(&xinf->lhs, **refp);
+			scannext(refp);
+		}
+		if (*refp && **refp == '"')
+			scannext(refp);
+	}
+	else
+	{
+		/* collect following characters up to next whitespace or '|' */
+		while (*refp && !isspace(**refp) && **refp != '|')
+		{
+			buildCHAR(&xinf->lhs, **refp);
+			scannext(refp);
+		}
 	}
 }
 
@@ -1541,6 +1561,8 @@ static BOOLEAN parsefileargs(refp, xinf, flags)
 {
 	CHAR	*filename;
 	CHAR	*scan;
+	CHAR	*expr, *val;
+	int	i;
 
 	/* if no filenames expected, or hit end of cmd, then do nothing */
 	if ((flags & (a_File|a_Files|a_Filter|a_Append)) == 0 || !*refp)
@@ -1589,7 +1611,9 @@ static BOOLEAN parsefileargs(refp, xinf, flags)
 	}
 
 	/* collect each whitespace-delimited filename argument */
-	for (filename = NULL; *refp && **refp != '|' && **refp != '\n'; scannext(refp))
+	for (filename = val = NULL;
+	     *refp && **refp != '|' && **refp != '\n';
+	     scannext(refp))
 	{
 		/* if whitespace, then process filename (if any) and then skip */
 		switch (**refp)
@@ -1622,7 +1646,7 @@ static BOOLEAN parsefileargs(refp, xinf, flags)
 				 */
 				scanprev(refp);
 			}
-			else if (!CHARchr(toCHAR(" \t\\#!*"), **refp))
+			else if (!CHARchr(toCHAR(" \t\\#!*`"), **refp))
 			{
 				/* backslash isn't followed by a char that might
 				 * require backslash quoting, so the backslash
@@ -1674,6 +1698,74 @@ static BOOLEAN parsefileargs(refp, xinf, flags)
 			msg(MSG_ERROR, "NUL not allowed in file name");
 			goto Error;
 
+#ifdef FEATURE_BACKTICK
+		  case '`':
+			/* build an expression which reads from a program */
+			expr = NULL; 
+			buildstr(&expr, "shell(\"");
+			scannext(refp);
+			for (; *refp && **refp && **refp != '|' && **refp != '\n' && **refp != '`'; scannext(refp))
+			{
+				if (**refp == '\\')
+				{
+					if (!scannext(refp) || !**refp || **refp == '\n')
+					{
+						msg(MSG_ERROR, "unterminated `prog` expression");
+						goto Error;
+					}
+					if (**refp != '`' || **refp != '\\')
+						buildCHAR(&expr, '\\');
+				}
+				else if (**refp == '"')
+					buildCHAR(&expr, '\\');
+				buildCHAR(&expr, **refp);
+			}
+			buildstr(&expr, "\")");
+
+			/* evaluate the expression */
+			val = calculate(expr, NULL, False);
+			safefree(expr);
+			if (!val)
+				/* error message already given */
+				goto Error;
+
+			/* resume previous filename, if any */
+			if (filename)
+			{
+				expr = filename;
+				filename = NULL;
+				for (i = 0; i < CHARlen(expr); i++)
+					buildCHAR(&filename, expr[i]);
+				safefree(expr);
+			}
+
+			/* Find whitespace-delimited names.  Treat any special
+			 * chars as normal chars.
+			 */
+			for (; *val; val++)
+			{
+				if (isspace(*val))
+				{
+					if (filename)
+					{
+						/* store the name */
+						if (!exaddfilearg(&xinf->file, &xinf->nfiles, tochar8(filename), True))
+							goto Error;
+
+						/* free the buildCHAR copy of the name */
+						safefree(filename);
+						filename = NULL;
+					}
+				}
+				else
+					buildCHAR(&filename, *val);
+			}
+			/* NOTE: val (returned by calculate()) is static so
+			 * we don't need to worry about freeing it.
+			 */
+			break;
+#endif /* FEATURE_BACKTICK */
+
 		  default:
 			/* Append the character to the name.  If this results
 			 * in a partial name of ">>" and either this isn't the
@@ -1759,8 +1851,8 @@ static RESULT parse(win, refp, xinf)
 	CHAR	*logstr;/* start of string command, or NULL if from buffer */
 	int	i;
 
-	/* if verbose, then display this line on stderr */
-	if (o_verbose >= 3 && !win)
+	/* if verbose, then display this line in window, or on stderr */
+	if (o_verbose >= (win ? 5 : 3))
 	{
 		lntext = NULL;
 		for (scandup(&p2, refp); p2 && *p2 != '\n'; scannext(&p2))
@@ -2463,6 +2555,9 @@ RESULT experform(win, top, bottom)
 	/* for each command... */
 	while (p && markoffset(top) < markoffset(bottom))
 	{
+		/* remember the location, for error reporting */
+		msgscriptline(top, NULL);
+
 		/* parse an ex command */
 		switch (parse(win, &p, &xinfb))
 		{
@@ -2510,21 +2605,19 @@ More:
 /* This function resembles experform(), except that this function parses
  * from a string instead of from a buffer.
  */
-RESULT exstring(win, str)
+RESULT exstring(win, str, name)
 	WINDOW	win;	/* default window (implies default buffer) */
 	CHAR	*str;	/* the string containing ex commands */
+	char	*name;	/* name for error reporting, or NULL if not known */
 {
 	EXINFO	xinfb;	/* buffer, holds info about command being parsed */
 	CHAR	*p;	/* pointer used for scanning command line */
-	BOOLEAN	oldthenflag;
-	CHAR	*olddotest;
+	EXCTLSTATE oldctlstate;
 	RESULT	result;
 	void	*locals;
 
-	/* commands run this way don't affect the :if or :while expressions */
-	oldthenflag = exthenflag;
-	olddotest = exdotest;
-	exdotest = NULL;
+	/* commands run this way don't affect :if/:while/:switch expressions */
+	exctlsave(oldctlstate);
 	locals = optlocal(NULL);
 
 	/* start reading commands */
@@ -2533,6 +2626,9 @@ RESULT exstring(win, str)
 	/* for each command... */
 	while (p && *p)
 	{
+		/* remember its location, for error reporting */
+		msgscriptline(scanmark(&p), name);
+
 		/* parse and execute one ex command */
 		if (parse(win, &p, &xinfb) != RESULT_COMPLETE
 		 || execute(&xinfb) != RESULT_COMPLETE)
@@ -2552,8 +2648,7 @@ Fail:
 
 Done:
 	(void)optlocal(locals);
-	exthenflag = oldthenflag;
-	exdotest = olddotest;
+	exctlrestore(oldctlstate);
 	return result;
 }
 
@@ -2741,6 +2836,13 @@ long exprintlines(win, mark, qty, pflag)
 
 
 #ifdef FEATURE_COMPLETE
+/* Return the remainder of a command name, buffer name, or tag name.  If file
+ * name completion is appropriate, then return NULL.  If nothing can be
+ * completed then just return a string containing a single tab character.
+ *
+ * This function may turn on the completebinary option.  It does this when
+ * it returns NULL (to denote file name completion) in a shell command line.
+ */
 CHAR *excomplete(win, from, to)
 	WINDOW	win;	/* window where multiple matches are listed */
 	MARK	from;	/* start of the line where completion is needed */
@@ -2754,6 +2856,7 @@ CHAR *excomplete(win, from, to)
 	CHAR	*cmdname;
 	int	len, mlen;
 	int	i, j, match;
+	BOOLEAN	filter;
 
 	(void)scanalloc(&s, from);
 	offset = markoffset(from);
@@ -2790,7 +2893,10 @@ CHAR *excomplete(win, from, to)
 	if (!s || offset >= markoffset(to) || !isalpha(*s))
 	{
 		if (s && offset < markoffset(to) && *s == '!')
+		{
+			o_completebinary = True;
 			cmdname = NULL;
+		}
 		else
 			cmdname = tab;
 		scanfree(&s);
@@ -2804,6 +2910,17 @@ CHAR *excomplete(win, from, to)
 		len = buildCHAR(&cmdname, *s);
 		offset++;
 	} while (scannext(&s) && offset < markoffset(to) && isalpha(*s));
+
+	/* if followed by whitespace and then a !, then remember that */
+	filter = False;
+	if (s && isspace(*s))
+	{
+		while (scannext(&s) && isspace(*s))
+		{
+		}
+		if (s && *s == '!')
+			filter = True;
+	}
 	scanfree(&s);
 
 	/* if cursor is at end of command name, then try to expand the name.
@@ -2912,13 +3029,16 @@ CHAR *excomplete(win, from, to)
 	if (cmdnames[i].code == EX_SET)
 		return optcomplete(win, to);
 
-	/* The :tag and :stag commands complete tag tags.  I'm tempted to
-	 * add :browse and :sbrowse here, but I suspect that filename
-	 * completion might be more useful for them.
+	/* The :tag and :stag commands complete tag names.  I'm tempted to
+	 * add :browse and :sbrowse here, but that might be confusing.
 	 */
 	if (cmdnames[i].code == EX_TAG || cmdnames[i].code == EX_STAG)
 		return tagcomplete(win, to);
 
+	/* The :cc and :make commands take filename arguments. */
+	if (cmdnames[i].code == EX_CC || cmdnames[i].code == EX_MAKE)
+		return NULL; /* do filename expansion */
+
 	/* Many commands take ex commands as their arguments.  The a_Cmds
 	 * flag is set for these commands, but it is also set for a few
 	 * others.  Oh well, it's close enough.  Also, the :help command's
@@ -2929,7 +3049,11 @@ CHAR *excomplete(win, from, to)
 
 	/* does the command take filename arguments or a target address? */
 	if ((cmdnames[i].flags & (a_Target | a_Filter | a_File | a_Files)) != 0)
+	{
+		if (filter)
+			o_completebinary = True;
 		return NULL;/* do filename expansion */
+	}
 
 	/* if all else fails, just treat it literally */
 	return tab;
