@@ -1,7 +1,7 @@
 /* exaction.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_exaction[] = "$Id: exaction.c,v 2.84 1998/10/06 16:11:02 steve Exp $";
+char id_exaction[] = "$Id: exaction.c,v 2.88 1999/10/07 16:23:58 steve Exp $";
 
 #include "elvis.h"
 
@@ -214,7 +214,7 @@ RESULT ex_all(xinf)
 			xinf->window->cursor = marktmp(bufline1, buf, 0);
 
 		/* execute the command */
-		result = exstring(xinf->window, xinf->rhs);
+		result = exstring(xinf->window, xinf->rhs, NULL);
 	}
 
 	/* Restore the original cursor */
@@ -533,10 +533,13 @@ RESULT	ex_file(xinf)
 	switch (xinf->command)
 	{
 	  case EX_EQUAL:
-		if (xinf->from != xinf->to)
+		if (xinf->anyaddr)
 		{
-			msg(MSG_INFO, "[ddd]$1,$2 = $3 lines",
-			       xinf->from, xinf->to, xinf->to - xinf->from + 1);
+			if (xinf->from != xinf->to)
+				msg(MSG_INFO, "[dd]$1,$2 = ($2-$1+1) lines",
+				       xinf->from, xinf->to);
+			else
+				msg(MSG_INFO, "[d]$1", xinf->from);
 		}
 		else
 		{
@@ -634,7 +637,9 @@ RESULT	ex_mark(xinf)
 	}
 
 	/* set the mark */
-	namedmark[*xinf->lhs - 'a'] = markdup(xinf->fromaddr);
+	namedmark[*xinf->lhs - 'a'] = markdup(xinf->anyaddr || !xinf->window
+						? xinf->fromaddr
+						: xinf->window->cursor);
 	return RESULT_COMPLETE;
 }
 
@@ -960,7 +965,7 @@ RESULT	ex_bang(xinf)
 
 	/* clean up & exit */
 	markfree(mark);
-	return (prgclose() == 0) ? RESULT_COMPLETE : RESULT_ERROR;
+	return (gui->prgclose ? (*gui->prgclose)() : prgclose()) == 0 ? RESULT_COMPLETE : RESULT_ERROR;
 }
 
 
@@ -999,6 +1004,8 @@ RESULT	ex_source(xinf)
 	/* create a temp buffer */
 	buf = bufalloc(NULL, 0, True);
 	assert(buf != NULL);
+	o_filename(buf) = CHARdup(toCHAR(xinf->file[0])); 
+	optflags(o_filename(buf)) |= OPT_FREE;
 
 	/* fill the temp buffer with text read from the file */
 	io = (CHAR *)safealloc(1024, sizeof(CHAR));
@@ -1162,7 +1169,7 @@ RESULT	ex_browse(xinf)
 	assert(xinf->command == EX_BROWSE || xinf->command == EX_SBROWSE);
 
 	/* build the tags document */
-	if (o_tagprg || o_tagprgonce)
+	if (o_tagprg)
 	{
 		/* using an external search program */
 		buf = tebrowse(xinf->bang, xinf->rhs);
