@@ -73,7 +73,7 @@ LONG gwframe_WM_INITMENU (GUI_WINDOW *gwp, UINT wParam, LONG lParam)
     /* check if syntax mode is active */
     hMenu = GetMenu (gwp->frameHWnd);
     if (hMenu == (HMENU)wParam) {
-        if (strcmp (optgetstr ("display", NULL), "syntax") == 0)
+        if (windefault->md == &dmsyntax)
             EnableMenuItem (hMenu, IDM_OPTIONS_SYNTAX, MF_ENABLED);
         else
             EnableMenuItem (hMenu, IDM_OPTIONS_SYNTAX, MF_GRAYED);
@@ -522,12 +522,13 @@ LONG gwclient_WM_MOUSEMOVE (GUI_WINDOW *gwp, UINT wParam, LONG lParam)
 LONG gwclient_WM_MOUSEWHEEL (GUI_WINDOW *gwp, UINT wParam, LONG lParam)
 
 {
-    long        delta = (HIWORD (wParam) / WHEEL_DELTA) * 4;
+    short        delta = HIWORD (wParam);
 
+    delta = (delta / WHEEL_DELTA) * 4;
     if (delta > 0)
-        eventscroll ((GUIWIN *)gwp, SCROLL_FWDLN, delta, 0L);
+        eventscroll ((GUIWIN *)gwp, SCROLL_BACKLN, delta, 0L);
     else
-        eventscroll ((GUIWIN *)gwp, SCROLL_BACKLN, -delta, 0L);
+        eventscroll ((GUIWIN *)gwp, SCROLL_FWDLN, -delta, 0L);
     gw_redraw_win (gwp);
     
     return 0;
@@ -774,4 +775,4 @@ LONG gwclient_WM_VSCROLL (GUI_WINDOW *gwp, UINT wParam, LONG lParam)
 }
 
 #endif
-
+/* ex:se ts=4 sw=4: */
