@@ -2,7 +2,7 @@
 
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_guix11[] = "$Id: guix11.c,v 2.34 1998/11/21 01:42:04 steve Exp $";
+char id_guix11[] = "$Id: guix11.c,v 2.39 1999/10/06 19:11:37 steve Exp $";
 
 #include "elvis.h"
 #ifdef GUI_X11
@@ -99,6 +99,9 @@ static void flush P_((void));
 static BOOLEAN guicmd P_((GUIWIN *gw, char *extra));
 static int xoptisfont P_((OPTDESC *desc, OPTVAL *val, CHAR *newval));
 static int xoptisnumber P_((OPTDESC *desc, OPTVAL *val, CHAR *newval));
+#ifndef NO_XLOCALE
+static XIC createic P_((Window window));
+#endif
 
 Display		*x_display;		/* X11 display */
 int		x_screen;		/* screen number */
@@ -149,16 +152,27 @@ static struct
 } keys[] =
 {
 	{ "<Up>",	XK_Up,		"k",	MAP_ALL },
+	{ "<KP_Up>",	XK_KP_Up,	"k",	MAP_ALL },
 	{ "<Down>",	XK_Down,	"j",	MAP_ALL },
+	{ "<KP_Down>",	XK_KP_Down,	"j",	MAP_ALL },
 	{ "<Left>",	XK_Left,	"h",	MAP_ALL },
+	{ "<KP_Left>",	XK_KP_Left,	"h",	MAP_ALL },
 	{ "<Right>",	XK_Right,	"l",	MAP_ALL },
+	{ "<KP_Right>",	XK_KP_Right,	"l",	MAP_ALL },
 	{ "<Prior>",	XK_Prior,	"\002",	MAP_ALL_VISUAL },
+	{ "<KP_Prior>",	XK_KP_Prior,	"\002",	MAP_ALL_VISUAL },
 	{ "<Next>",	XK_Next,	"\006",	MAP_ALL_VISUAL },
+	{ "<KP_Next>",	XK_KP_Next,	"\006",	MAP_ALL_VISUAL },
 	{ "<Home>",	XK_Home,	"^",	MAP_ALL },
+	{ "<KP_Home>",	XK_KP_Home,	"^",	MAP_ALL },
 	{ "<Begin>",	XK_Begin,	"^",	MAP_ALL },
+	{ "<KP_Begin>",	XK_KP_Begin,	"^",	MAP_ALL },
 	{ "<End>",	XK_End,		"$",	MAP_ALL },
+	{ "<KP_End>",	XK_KP_End,	"$",	MAP_ALL },
 	{ "<Insert>",	XK_Insert,	"i",	MAP_ALL },
+	{ "<KP_Insert>",XK_KP_Insert,	"i",	MAP_ALL },
 	{ "<Delete>",	XK_Delete,	"x",	MAP_ALL },
+	{ "<KP_Delete>",XK_KP_Delete,	"x",	MAP_ALL },
 	{ "<Undo>",	XK_Undo,	"u",	MAP_ALL_VISUAL },
 	{ "<Help>",	XK_Help,	":help\r",MAP_ALL_VISUAL },
 	{ "<Multi_key>",XK_Multi_key,	"\013",	MAP_INPUT|MAP_OPEN }
@@ -196,6 +210,8 @@ static OPTDESC x11desc[] =
 	{"underline", "uln",	NULL,		NULL		},
 	{"outlinemono", "om",	optnstring,	optisnumber,	"0:3"},
 	{"borderwidth", "xbw",	optnstring,	xoptisnumber,	"0:5"},
+	{"xrootwidth", "xrw",	optnstring,	optisnumber,	"320:4096"},
+	{"xrootheight", "xrh",	optnstring,	optisnumber,	"200:4096"},
 	{"submit", "Submit",	optsstring,	optisstring	},
 	{"cancel", "Cancel",	optsstring,	optisstring	},
 	{"help", "Help",	optsstring,	optisstring	}
@@ -985,6 +1001,8 @@ static int init(argc, argv)
 	optpreset(o_underline, True, OPT_HIDE|OPT_SCRATCH);
 	optpreset(o_outlinemono, DEFAULT_OUTLINEMONO, OPT_HIDE);
 	optpreset(o_borderwidth, DEFAULT_BORDERWIDTH, OPT_HIDE);
+	optpreset(o_xrootwidth, rootwidth, OPT_HIDE|OPT_LOCK);
+	optpreset(o_xrootheight, rootheight, OPT_HIDE|OPT_LOCK);
 	optinsert("x11", QTY(x11desc), x11desc, (OPTVAL *)&x_optvals);
 
 	/* initialize the colors */
@@ -1141,7 +1159,8 @@ static void destroygw(gw, force)
 	x_st_destroy(xw);
 	XFreeGC(x_display, xw->gc);
 #ifndef NO_XLOCALE
-	XDestroyIC(xw->ic);
+	if (xw->ic)
+		XDestroyIC(xw->ic);
 #endif
 	if (xw->win)
 	{
@@ -1399,7 +1418,7 @@ static void term()
 }
 
 #ifndef NO_XLOCALE
-/* create an input context for a given window.  The first time this function is
+/* Create an input context for a given window.  The first time this function is
  * called, it will also initialize a few static variables.
  */
 static XIC createic(window)
@@ -1408,21 +1427,24 @@ static XIC createic(window)
 	char	*p;
 	XIMStyles *xim_styles = NULL;
 	int	found;
-	XIM	xim = NULL;
+ static XIM	xim = NULL;
  static XIMStyle input_style = 0;
  static int	firsttime = True;
+	int	i;
 
 	if (firsttime)
 	{
-		if (((p = XSetLocaleModifiers ("@im=none")) != NULL && *p)
-		 || ((p = XSetLocaleModifiers ("")) != NULL && *p))
-			xim = XOpenIM (Xdisplay, NULL, NULL, NULL);
+		/* Open the input method. */
+		if (((p = XSetLocaleModifiers("@im=none")) != NULL && *p)
+		 || ((p = XSetLocaleModifiers("")) != NULL && *p))
+			xim = XOpenIM(x_display, NULL, NULL, NULL);
 		if (xim == NULL)
 		{
 			/* Failed to open input method */
 			return NULL;
 		}
 
+		/* Check for input styles */
 		if (XGetIMValues(xim, XNQueryInputStyle, &xim_styles, NULL)
 		 || !xim_styles)
 		{
@@ -1435,13 +1457,13 @@ static XIC createic(window)
 		input_style = (XIMPreeditNothing | XIMStatusNothing);
 		for (i = 0; i < xim_styles->count_styles; i++)
 		{
-			if (input_style == xim_styles->supported_styles [i])
+			if (input_style == xim_styles->supported_styles[i])
 			{
 				found = 1;
 				break;
 			}
 		}
-		XFree (xim_styles);
+		XFree(xim_styles);
 		if (found == 0)
 		{
 			/* input method doesn't support my preedit type */
@@ -1454,7 +1476,7 @@ static XIC createic(window)
 	}
 
 	/* create the context */
-	return XCreateIC (xim, XNInputStyle, input_style,
+	return XCreateIC(xim, XNInputStyle, input_style,
 			      XNClientWindow, window,
 			      XNFocusWindow, window,
 			      NULL);
@@ -1629,7 +1651,7 @@ static BOOLEAN creategw(name, firstcmd)
 
 #ifndef NO_XLOCALE
 	/* give it an input context */
-	xw->ic = createic(xw);
+	xw->ic = createic(xw->win);
 #endif
 
 	/* input event selection */
@@ -1649,7 +1671,7 @@ static BOOLEAN creategw(name, firstcmd)
 	if (firstcmd)
 	{
 		winoptions(winofgw((GUIWIN *)xw));
-		exstring(windefault, toCHAR(firstcmd));
+		exstring(windefault, toCHAR(firstcmd), "+cmd");
 	}
 
 	return True;
