@@ -1,7 +1,7 @@
 /* lpovrtyp.c */
 /* Copyright 1995 by Steve Kirkendall */
 
-char id_lpovrtyp[] = "$Id: lpovrtyp.c,v 2.10 1996/05/24 00:56:28 steve Exp $";
+char id_lpovrtyp[] = "$Id: lpovrtyp.c,v 2.12 1997/10/05 19:06:12 steve Exp $";
 
 
 /* This file contains a driver for printer types which use overtyping to
@@ -13,6 +13,7 @@ char id_lpovrtyp[] = "$Id: lpovrtyp.c,v 2.10 1996/05/24 00:56:28 steve Exp $";
  */
 
 #include "elvis.h"
+#ifdef FEATURE_LPR
 
 #if USE_PROTOTYPES
 static void outcr(void);
@@ -201,5 +202,7 @@ static void after(linesleft)
 }
 
 /* These describe the printer types supported by these functions */
-LPTYPE lpcr =	{"cr", 0, before, fontch, page, after};
-LPTYPE lpbs =	{"bs", 1, before, fontch, page, after};
+LPTYPE lpcr =	{"cr", 0, True, before, fontch, page, after};
+LPTYPE lpbs =	{"bs", 1, True, before, fontch, page, after};
+
+#endif /* FEATURE_LPR */
