@@ -24,6 +24,7 @@ typedef unsigned int MAPFLAGS;
 #define MAP_DISABLE	0x0400	/* disable all maps for next keystroke */
 #define MAP_BREAK	0x0800	/* switch from "run" to "step" trace mode */
 
+BEGIN_EXTERNC
 extern void	mapinsert P_((CHAR *rawin, int rawlen, CHAR *cooked, int cooklen, CHAR *label, MAPFLAGS flags));
 extern BOOLEAN	mapdelete P_((CHAR *rawin, int rawlen, MAPFLAGS flags, BOOLEAN del, BOOLEAN brk));
 extern MAPSTATE	mapdo P_((CHAR *keys, int nkeys));
@@ -31,8 +32,10 @@ extern void	mapunget P_((CHAR *keys, int nkeys, BOOLEAN remap));
 extern CHAR	*maplist P_((MAPFLAGS flags, int *reflen));
 extern RESULT	maplearn P_((_CHAR_ buf, BOOLEAN starting));
 extern CHAR	maplrnchar P_((_CHAR_ dflt));
+extern BOOLEAN	mapbusy P_((void));
 extern void	mapalert P_((void));
 extern CHAR	*mapabbr P_((CHAR *bkwd, long *oldptr, long *newptr, BOOLEAN exline));
 #ifndef NO_EXRC
 extern void	mapsave P_((BUFFER buf));
 #endif
+END_EXTERNC
