@@ -2,6 +2,7 @@
 /* Copyright 1995 by Steve Kirkendall */
 
 
+BEGIN_EXTERNC
 extern RESULT	m_left P_((WINDOW win, VIINFO *vinf));
 extern RESULT	m_right P_((WINDOW win, VIINFO *vinf));
 extern RESULT	m_updown P_((WINDOW win, VIINFO *vinf));
@@ -23,3 +24,8 @@ extern RESULT	m_bsentence P_((WINDOW win, VIINFO *vinf));
 
 /* in search.c */
 extern RESULT	m_search P_((WINDOW win, VIINFO *vinf));
+extern regexp	*searchre;
+extern BOOLEAN	searchforward;
+extern BOOLEAN	searchhasdelta;
+extern long	searchdelta;
+END_EXTERNC
