@@ -18,7 +18,11 @@
 #include <errno.h>
 #ifdef  __TURBOC__
 # include <dir.h>
-# define ff_name name
+extern unsigned _stklen = 16384U;
+#endif
+#ifdef GO32
+# include <dir.h>
+# include <fcntl.h>
 #endif
 #ifdef M_I86
 # include <direct.h>
@@ -38,6 +42,10 @@
 # define find_t		_find_t
 # define stat		_stat
 #endif
+#ifndef A_RDONLY
+# define A_RDONLY	_A_RDONLY
+# define A_SUBDIR	_A_SUBDIR
+#endif
 
 #ifndef NAME_MAX
 # ifdef MAXNAMLEN
@@ -47,6 +55,10 @@
 # endif
 #endif
 
+
+static char *lastslash(char *str);
+
+
 #if !defined(JUST_DIRPATH)
 /* These are used for communication between dirfirst() and dirnext() */
 static char finddir[NAME_MAX];
@@ -71,15 +83,16 @@ char *dirfirst(char *wildexpr, BOOLEAN ispartial)
 		strcat(findwild, strchr(findwild, '.') ? "*" : "*.*");
 
 	/* if no match, then return the original wildexpr unchanged */
-	if (findfirst(findwild, &ff, 0) != 0)
+	if (findfirst(findwild, &ff, A_RDONLY|A_SUBDIR) != 0)
 	{
 		found[0] = '\0';
 		return wildexpr;
 	}
 
 	/* combine the directory name with the found file's name */
-	strcpy(found, dirpath(finddir, ff.name));
+	strcpy(found, dirpath(finddir, ff.ff_name));
 
+#ifndef JUST_DIRFIRST
 	/* convert the filename to lowercase */
 	for (wildexpr = found + strlen(found) - 1;
 	     wildexpr >= found && *wildexpr != '\\' && *wildexpr != ':';
@@ -87,6 +100,7 @@ char *dirfirst(char *wildexpr, BOOLEAN ispartial)
 	{
 		*wildexpr = tolower(*wildexpr);
 	}
+#endif
 	return found;
 }
 
@@ -109,8 +123,9 @@ char *dirnext(void)
 	}
 
 	/* combine the directory name with the found file's name */
-	strcpy(found, dirpath(finddir, ff.name));
+	strcpy(found, dirpath(finddir, ff.ff_name));
 
+#ifndef JUST_DIRFIRST
 	/* convert the filename to lowercase */
 	for (scan = found + strlen(found) - 1;
 	     scan >= found && *scan != '\\' && *scan != ':';
@@ -118,6 +133,7 @@ char *dirnext(void)
 	{
 		*scan = tolower(*scan);
 	}
+#endif
 	return found;
 }
 #endif /* !JUST_DIRPATH */
@@ -165,7 +181,20 @@ DIRPERM dirperm(char *filename)
 }
 #endif /* !JUST_DIRFIRST && !JUST_DIRPATH */
 
-#if !defined(JUST_DIRPATH)
+static char *lastslash(char *str)
+{
+	char	*slash;
+
+	for (slash = &str[strlen(str)];
+	     slash >= str && *slash != '/' && *slash != '\\';
+	     slash--)
+	{
+	}
+	if (slash < str)
+		slash = NULL;
+	return slash;
+}
+
 /* return the directory part of a pathname */
 char *dirdir(char *pathname)
 {
@@ -173,7 +202,7 @@ char *dirdir(char *pathname)
 	char	*slash;
 
 	strcpy(ret, pathname);
-	slash = strrchr(ret, '\\');
+	slash = lastslash(ret);
 	if (slash == ret)
 	{
 		ret[1] = '\0';
@@ -196,7 +225,6 @@ char *dirdir(char *pathname)
 	}
 	return ret;
 }
-#endif /* !JUST_DIRPATH */
 
 #if !defined(JUST_DIRFIRST) && !defined(JUST_DIRPATH)
 /* return the file part of a pathname.  This particular implementation doesn't
@@ -207,7 +235,7 @@ char *dirfile(char *pathname)
 {
 	char	*slash;
 
-	slash = strrchr(pathname, '\\');
+	slash = lastslash(pathname);
 	if (!slash && isalpha(*pathname) && pathname[1] == ':')
 	{
 		slash = &pathname[1];
@@ -231,9 +259,12 @@ char *dirpath(char *dir, char *file)
 		 */
 		strcpy(path, file);
 	}
-	else if (isalpha(dir[0]) && dir[1] == ':' && file[0] == '\\')
+	else if (isalpha(dir[0]) && dir[1] == ':'
+				&& (!dir[2] || file[0] == '\\'))
 	{
-		/* dir has drive letter, and file is absolute within drive */
+		/* dir has drive letter, and either dir has no directory name
+		 * or file is absolute within drive.
+		 */
 		sprintf(path, "%.2s%s", dir, file);
 	}
 	else if (!dir[0] || dir[strlen(dir) - 1] != '\\')
