@@ -35,10 +35,13 @@
 # include <dos.h>
 #else
 # ifndef REGS
-#  define REGS	_REGS
+#  ifndef GO32
+#   define REGS	_REGS
+#  endif
 # endif
 #endif
 
+BEGIN_EXTERNC
 #if USE_PROTOTYPES
 static void v_up(void);
 static void v_cb(void);
@@ -57,6 +60,7 @@ static int v_rows(void);
 #endif
 
 static void video P_((int, int *, int *));
+END_EXTERNC
 
 static BOOLEAN vmono;	/* is this a monochrome screen? */
 static int vmode = 0x07;/* attribute byte */
@@ -189,7 +193,6 @@ int v_put(ch)
 	ch &= 0xff;
 	if (ch >= ' ')
 		video(0x900 | ch, &cx, (int *)0);
-	video(0xe00 | ch, (int *)0, (int *)0);
 	return ch;
 }
 
@@ -222,6 +225,7 @@ int v_rows()
 {
 	int line, oldline;
 
+#ifndef GO32
 	/* screen size less then 4K? then we have 25 lines only */
 
 	if (*(int far *)(0x0040004cl)<=4096)
@@ -235,6 +239,7 @@ int v_rows()
 		(*(unsigned char far *)(0x00400084L)+1))+0xfff)&(~0xfff)) ==
 		*(unsigned int far *)(0x0040004cL))
 			return (vrows = *(unsigned char far *)(0x00400084L)+1);
+#endif
 
 	/* uh oh. Emit '\n's until screen starts scrolling. */
 
