@@ -6,6 +6,7 @@
 #ifndef OSNAME
 # define OSNAME	"unix"
 #endif
+#define ANY_UNIX 1
 
 /*=============================================================================
  * This is the default shell, as reported by ":set shell?"
@@ -41,6 +42,14 @@
  */
 #define OSSESSIONPATH	"/var/tmp:/tmp:~:."
 
+/* This is the default value of the "syntax" display mode's includepath option,
+ * which tells elvis where to look for #include files.  It can be overridden
+ * at run time by an INCLUDEPATH environment variable.
+ */
+#ifndef OSINCLUDEPATH
+# define OSINCLUDEPATH	"/usr/local/include:/usr/include"
+#endif
+
 /*=============================================================================
  * This is the name of a directory where elvis stores some default files.
  * It is incorporated into the default value of the "loadpath" option.  This
@@ -72,6 +81,11 @@
  */
 #define TTY_DEFAULT	"unknown"
 
+/*=============================================================================
+ * This determines whether filename completion should ignore case differences.
+ */
+#define FILES_IGNORE_CASE 0
+
 /*=============================================================================
  * Miscellaneous tweaks
  */
@@ -80,15 +94,34 @@
 # define NEED_STRDUP
 #endif
 
-#if defined(M_XENIX) && defined(__STDC__)
+/* Some newer Linux systems have speed_t defined in <termios.h> instead of
+ * the traditional <sys/types.h>.
+ */
+#if defined(linux) && !defined(USE_TERMIO) && !defined(USE_SGTTY)
+# include <termios.h>
+#endif
+
+#if (defined(M_XENIX) || defined(__QNX__)) && defined(__STDC__)
 extern char	PC;		/* Pad char */
 extern char	*BC;		/* backspace */
 extern char	*UP;		/* cursor up */
 extern short	ospeed;		/* tty speed, eg B2400 */
+
+# if defined (__cplusplus)
+extern "C" {
+# endif
+# ifdef __QNX__
+#  include <termcap.h>
+# else
 extern int	tgetent(char *, char *);
 extern int	tgetnum(char *);
 extern int	tgetflag(char *);
 extern char	*tgoto(char *, int, int);
 extern char	*tgetstr(char*, char**);
 extern void	tputs(char *, int, int (*)(int));
+# endif
+# if defined (__cplusplus)
+}
+# endif
+
 #endif
