@@ -8,13 +8,14 @@
  * defined then the whole file is compiled.
  */
 
-char id_osdir[] = "$Id: osdir.c,v 2.12 1996/05/23 18:52:33 steve Exp $";
+char id_osdir[] = "$Id: osdir.c,v 2.19 1998/02/16 23:55:53 steve Exp $";
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <errno.h>
 #include <dirent.h>
+#include <pwd.h>
 #ifndef S_ISREG
 # define S_ISREG(mode)	(((mode) & 0070000) == 0)
 #endif
@@ -187,8 +188,15 @@ char *dirnext()
 		return (char *)0;
 	}
 
-	/* loop until we find a matching entry */
-	while ((dent = readdir(dirfp)) != NULL && !wildcmp(dent->d_name, wildfile))
+	/* loop until we find a matching entry, or end of directory.
+	 * Note that we're careful about not allowing the ? and * wildcards
+	 * match a . at the start of a filename; those files are supposed to
+	 * be hidden.
+	 */
+
+	while ((dent = readdir(dirfp)) != NULL
+	    && ((dent->d_name[0] == '.' && (wildfile[0] != '.' && wildfile[0] != '['))
+		|| !wildcmp(dent->d_name, wildfile)))
 	{
 	}
 
@@ -252,6 +260,24 @@ DIRPERM dirperm(filename)
 	char	*filename;	/* name of file to check */
 {
 	struct stat st;
+	int	i;
+
+	/* check for a protocol */
+	for (i = 0; isalpha(filename[i]); i++)
+	{
+	}
+	if (i < 2 || filename[i] != ':')
+		i = 0;
+
+	/* Skip past "file:" protocol; assume ftp is read/write, and all others
+	 * are readonly
+	 */
+	if (!strncmp(filename, "file:", 5))
+		filename += i + 1;
+	else if (!strncmp(filename, "ftp:", 4))
+		return DIR_READWRITE;
+	else if (i > 0)
+		return DIR_READONLY;
 
 	if (stat(filename, &st) < 0)
 	{
@@ -278,26 +304,6 @@ DIRPERM dirperm(filename)
 	}
 }
 
-/* return the directory part of a pathname */
-char *dirdir(pathname)
-	char	*pathname;
-{
-	static char dir[NAME_MAX + 1];
-	char	*slash;
-
-	strncpy(dir, pathname, sizeof dir);
-	slash = strrchr(dir, '/');
-	if (slash)
-	{
-		*slash = '\0';
-		return dir;
-	}
-	else
-	{
-		return ".";
-	}
-}
-
 /* return the file part of a pathname.  This particular implementation doesn't
  * use an internal buffer; it simply returns a pointer to the filename at the
  * end of the pathname.
@@ -321,6 +327,30 @@ char *dirfile(pathname)
 #endif /* !JUST_DIRFIRST */
 #endif /* !JUST_DIRPATH */
 
+/* return the directory part of a pathname */
+char *dirdir(pathname)
+	char	*pathname;
+{
+	static char dir[NAME_MAX + 1];
+	char	*slash;
+
+	strncpy(dir, pathname, sizeof dir);
+	slash = strrchr(dir, '/');
+	if (slash == dir)
+	{
+		return "/";
+	}
+	else if (slash)
+	{
+		*slash = '\0';
+		return dir;
+	}
+	else
+	{
+		return ".";
+	}
+}
+
 #ifndef JUST_DIRFIRST
 
 /* combine a directory name and a filename, yielding a pathname. */
@@ -329,6 +359,7 @@ char *dirpath(dir, file)
 	char	*file;	/* filename */
 {
 	static char	path[NAME_MAX + 1];
+	int		len;
 
 	if (*file == '/' || !strcmp(dir, "."))
 	{
@@ -336,7 +367,10 @@ char *dirpath(dir, file)
 	}
 	else
 	{
-		sprintf(path, "%s/%s", dir, file);
+		len = strlen(dir);
+		if (len > 0 && dir[len - 1] == '/')
+			len--;
+		sprintf(path, "%.*s/%s", len, dir, file);
 	}
 	return path;
 }
@@ -365,5 +399,30 @@ BOOLEAN dirchdir(pathname)
 	return (BOOLEAN)(chdir(pathname) >= 0);
 }
 
+/* This only handles ~user, because the ~/dir is handled elsewhere. */
+char *expanduserhome(pathname, dest)
+	char *pathname;
+	char *dest;
+{
+	char *p;
+	struct passwd *u;
+
+	strcpy(dest, pathname+1);
+	if ((p = strchr(dest, '/'))) /* yes, ASSIGNMENT! */
+		*p = 0;
+
+	if ((u = getpwnam(dest))) /* yes, ASSIGNMENT! */
+	{
+		strcpy(dest, u->pw_dir);
+		if ((p = strchr(pathname, '/'))) /* yes, ASSIGNMENT! */
+			strcat(dest, p);
+		strcpy(pathname, dest);
+	} else
+	{
+		strcpy(dest, pathname);
+	}
+	
+	return(dest);
+}
 #endif /* !JUST_DIRPATH */
 #endif /* !JUST_DIRFIRST */
