@@ -287,6 +287,7 @@ static void lookup(tag)
 	long	taglnum;/* line number of number tag address, or 0 */
 	char	*tagline;/* text form of regexp tag address */
 	LINECLS	lc;	/* line classification */
+	int	len;
 	int	i;
 
 	/* open the file, or the "refs" file if the source file is unreadable */
@@ -321,6 +322,9 @@ static void lookup(tag)
 				t++;
 			*l++ = *t++;
 		}
+		len = (int)(l - tagline);
+		if (*t == '$')
+			len++;
 		*l = '\0';
 	}
 
@@ -328,7 +332,7 @@ static void lookup(tag)
 	for (lnum = 1, lc = LC_COMPLETE; (line = getline(fp)) != NULL; lnum++)
 	{
 		/* is this the tag definition? */
-		if (taglnum > 0 ? taglnum == lnum : !strcmp(tagline, line))
+		if (taglnum > 0 ? taglnum == lnum : !strncmp(tagline, line, len))
 		{
 			/* output the tag location */
 			if (!omit_comment_lines)
@@ -536,7 +540,7 @@ int main(argc, argv)
 
 	/* -h implies -a; nobody would want to browse a single tag */
 	if (output_html_browser)
-		output_all_matches = True;
+		output_all_matches = 1;
 	if (output_html_browser + output_tag_info + output_verbose_info > 1)
 	{
 		fprintf(stderr, "%s: can't mix -t, -v, and -h\n", argv[0]);
