@@ -4,10 +4,12 @@
 
 #ifndef DEBUG_ALLOC
 
+BEGIN_EXTERNC
 extern void *safealloc P_((int qty, size_t size));
-# define safekept	safealloc
 extern void safefree P_((void *ptr));
 extern char *safedup P_((char *str));
+END_EXTERNC
+# define safekept	safealloc
 #define safekdup	safedup
 # define safeterm()
 # define safeinspect()
@@ -19,10 +21,12 @@ extern char *safedup P_((char *str));
 # define safefree(ptr)		_safefree(__FILE__, __LINE__, ptr)
 # define safedup(str)		_safedup(__FILE__, __LINE__, False, str)
 # define safekdup(str)		_safedup(__FILE__, __LINE__, True, str)
+BEGIN_EXTERNC
 extern void *_safealloc P_((char *file, int line, BOOLEAN kept, int qty, size_t size));
 extern void _safefree P_((char *file, int line, void *ptr));
 extern char *_safedup P_((char *file, int line, BOOLEAN kept, char *str));
 extern void safeterm P_((void));
 extern void safeinspect P_((void));
+END_EXTERNC
 
 #endif
