@@ -2,6 +2,7 @@
 /* Copyright 1995 by Steve Kirkendall */
 
 
+BEGIN_EXTERNC
 extern RESULT v_at	P_((WINDOW win, VIINFO *vinf));
 extern RESULT v_delchar	P_((WINDOW win, VIINFO *vinf));
 extern RESULT v_ex	P_((WINDOW win, VIINFO *vinf));
@@ -17,3 +18,4 @@ extern RESULT v_tag	P_((WINDOW win, VIINFO *vinf));
 extern RESULT v_undo	P_((WINDOW win, VIINFO *vinf));
 extern RESULT v_visible	P_((WINDOW win, VIINFO *vinf));
 extern RESULT v_window	P_((WINDOW win, VIINFO *vinf));
+END_EXTERNC
